"""Tests for A2A agent card."""

from datetime import datetime, timezone

import pytest

from sardis_a2a.agent_card import (
    AgentCapability,
    PaymentCapability,
    ServiceEndpoint,
    SardisAgentCard,
    create_sardis_agent_card,
)


class TestAgentCapability:
    """Tests for AgentCapability enum."""

    def test_payment_capabilities(self):
        """Test payment capability values."""
        assert AgentCapability.PAYMENT_EXECUTE.value == "payment.execute"
        assert AgentCapability.PAYMENT_VERIFY.value == "payment.verify"
        assert AgentCapability.PAYMENT_REFUND.value == "payment.refund"

    def test_mandate_capabilities(self):
        """Test mandate capability values."""
        assert AgentCapability.MANDATE_INGEST.value == "mandate.ingest"
        assert AgentCapability.MANDATE_SIGN.value == "mandate.sign"

    def test_wallet_capabilities(self):
        """Test wallet capability values."""
        assert AgentCapability.WALLET_BALANCE.value == "wallet.balance"
        assert AgentCapability.WALLET_HOLD.value == "wallet.hold"

    def test_checkout_capabilities(self):
        """Test checkout capability values."""
        assert AgentCapability.CHECKOUT_CREATE.value == "checkout.create"
        assert AgentCapability.CHECKOUT_COMPLETE.value == "checkout.complete"

    def test_micropay_capability(self):
        """Test micropay capability value."""
        assert AgentCapability.X402_MICROPAY.value == "x402.micropay"


class TestPaymentCapability:
    """Tests for PaymentCapability."""

    def test_default_values(self):
        """Test default payment capability values."""
        cap = PaymentCapability()

        assert "USDC" in cap.supported_tokens
        assert "USDT" in cap.supported_tokens
        assert "PYUSD" in cap.supported_tokens
        assert "EURC" in cap.supported_tokens

        assert "base" in cap.supported_chains
        assert "polygon" in cap.supported_chains
        assert "ethereum" in cap.supported_chains
        assert "arbitrum" in cap.supported_chains
        assert "optimism" in cap.supported_chains

        assert cap.min_amount_minor == 100  # $1.00
        assert cap.max_amount_minor == 100_000_00  # $100,000.00

        assert cap.ap2_compliant is True
        assert cap.x402_compliant is True
        assert cap.ucp_compliant is True

    def test_custom_values(self):
        """Test custom payment capability values."""
        cap = PaymentCapability(
            supported_tokens=["USDC"],
            supported_chains=["base", "polygon"],
            min_amount_minor=500,
            max_amount_minor=50_000_00,
            ap2_compliant=True,
            x402_compliant=False,
            ucp_compliant=True,
        )

        assert cap.supported_tokens == ["USDC"]
        assert cap.supported_chains == ["base", "polygon"]
        assert cap.min_amount_minor == 500
        assert cap.max_amount_minor == 50_000_00
        assert cap.x402_compliant is False

    def test_to_dict(self):
        """Test serialization to dictionary."""
        cap = PaymentCapability(
            supported_tokens=["USDC", "USDT"],
            supported_chains=["base"],
            min_amount_minor=100,
            max_amount_minor=10_000_00,
        )

        data = cap.to_dict()

        assert data["supported_tokens"] == ["USDC", "USDT"]
        assert data["supported_chains"] == ["base"]
        assert data["min_amount_minor"] == 100
        assert data["max_amount_minor"] == 10_000_00
        assert data["ap2_compliant"] is True


class TestServiceEndpoint:
    """Tests for ServiceEndpoint."""

    def test_create_endpoint(self):
        """Test creating a service endpoint."""
        endpoint = ServiceEndpoint(
            url="https://api.sardis.sh/v2",
            protocol="https",
            auth_required=True,
            auth_type="bearer",
        )

        assert endpoint.url == "https://api.sardis.sh/v2"
        assert endpoint.protocol == "https"
        assert endpoint.auth_required is True
        assert endpoint.auth_type == "bearer"

    def test_default_values(self):
        """Test default endpoint values."""
        endpoint = ServiceEndpoint(url="https://example.com")

        assert endpoint.protocol == "https"
        assert endpoint.auth_required is False
        assert endpoint.auth_type is None

    def test_to_dict(self):
        """Test endpoint serialization."""
        endpoint = ServiceEndpoint(
            url="https://api.sardis.sh/v2",
            protocol="https",
            auth_required=True,
            auth_type="api_key",
        )

        data = endpoint.to_dict()

        assert data["url"] == "https://api.sardis.sh/v2"
        assert data["protocol"] == "https"
        assert data["auth_required"] is True
        assert data["auth_type"] == "api_key"


class TestSardisAgentCard:
    """Tests for SardisAgentCard."""

    @pytest.fixture
    def agent_card(self):
        """Create a sample agent card."""
        return SardisAgentCard(
            agent_id="agent_123",
            agent_name="Test Agent",
            agent_version="1.0.0",
            agent_description="A test agent",
            operator_name="Test Operator",
            operator_url="https://test.com",
            capabilities=[
                AgentCapability.PAYMENT_EXECUTE,
                AgentCapability.PAYMENT_VERIFY,
                AgentCapability.CHECKOUT_CREATE,
            ],
        )

    def test_create_agent_card(self, agent_card):
        """Test creating an agent card."""
        assert agent_card.agent_id == "agent_123"
        assert agent_card.agent_name == "Test Agent"
        assert agent_card.agent_version == "1.0.0"
        assert agent_card.operator_name == "Test Operator"
        assert len(agent_card.capabilities) == 3

    def test_default_capabilities(self):
        """Test default agent card capabilities."""
        card = SardisAgentCard(
            agent_id="agent_1",
            agent_name="Agent",
        )

        # Check default capabilities include core functions
        assert AgentCapability.PAYMENT_EXECUTE in card.capabilities
        assert AgentCapability.PAYMENT_VERIFY in card.capabilities
        assert AgentCapability.CHECKOUT_CREATE in card.capabilities

    def test_supports_capability(self, agent_card):
        """Test capability checking."""
        assert agent_card.supports_capability(AgentCapability.PAYMENT_EXECUTE) is True
        assert agent_card.supports_capability(AgentCapability.PAYMENT_VERIFY) is True
        assert agent_card.supports_capability(AgentCapability.CHECKOUT_CREATE) is True
        assert agent_card.supports_capability(AgentCapability.PAYMENT_REFUND) is False

    def test_supports_token(self, agent_card):
        """Test token support checking."""
        # Default PaymentCapability includes USDC, USDT, etc.
        assert agent_card.supports_token("USDC") is True
        assert agent_card.supports_token("usdc") is True  # Case insensitive
        assert agent_card.supports_token("USDT") is True
        assert agent_card.supports_token("UNKNOWN") is False

    def test_supports_chain(self, agent_card):
        """Test chain support checking."""
        # Default PaymentCapability includes base, polygon, etc.
        assert agent_card.supports_chain("base") is True
        assert agent_card.supports_chain("BASE") is True  # Case insensitive
        assert agent_card.supports_chain("polygon") is True
        assert agent_card.supports_chain("unknown") is False

    def test_to_dict(self, agent_card):
        """Test agent card serialization."""
        data = agent_card.to_dict()

        assert data["agent_id"] == "agent_123"
        assert data["name"] == "Test Agent"
        assert data["version"] == "1.0.0"
        assert data["description"] == "A test agent"
        assert data["operator"]["name"] == "Test Operator"
        assert data["operator"]["url"] == "https://test.com"
        assert "payment.execute" in data["capabilities"]
        assert "payment" in data
        assert "endpoints" in data

    def test_to_dict_with_endpoints(self):
        """Test serialization with endpoints."""
        card = SardisAgentCard(
            agent_id="agent_123",
            agent_name="Test Agent",
            api_endpoint=ServiceEndpoint(
                url="https://api.test.com",
                auth_required=True,
                auth_type="bearer",
            ),
            mcp_endpoint="npx @sardis/mcp-server",
            a2a_endpoint=ServiceEndpoint(
                url="https://api.test.com/a2a",
                auth_required=True,
                auth_type="signature",
            ),
        )

        data = card.to_dict()

        assert "api" in data["endpoints"]
        assert data["endpoints"]["api"]["url"] == "https://api.test.com"
        assert data["endpoints"]["mcp"] == "npx @sardis/mcp-server"
        assert "a2a" in data["endpoints"]

    def test_to_dict_with_signing(self):
        """Test serialization with signing key."""
        card = SardisAgentCard(
            agent_id="agent_123",
            agent_name="Test Agent",
            signing_key_id="key_1",
            public_key="base64_encoded_key",
            key_algorithm="Ed25519",
        )

        data = card.to_dict()

        assert data["signing"] is not None
        assert data["signing"]["key_id"] == "key_1"
        assert data["signing"]["public_key"] == "base64_encoded_key"
        assert data["signing"]["algorithm"] == "Ed25519"

    def test_timestamps(self):
        """Test timestamp fields."""
        card = SardisAgentCard(
            agent_id="agent_1",
            agent_name="Agent",
        )

        assert isinstance(card.created_at, datetime)
        assert isinstance(card.updated_at, datetime)
        assert card.created_at.tzinfo == timezone.utc


class TestCreateSardisAgentCard:
    """Tests for create_sardis_agent_card helper."""

    def test_create_basic_card(self):
        """Test creating a basic agent card."""
        card = create_sardis_agent_card(
            agent_id="agent_test",
            agent_name="Test Agent",
            api_base_url="https://api.sardis.sh",
        )

        assert card.agent_id == "agent_test"
        assert card.agent_name == "Test Agent"
        assert card.agent_description == "Sardis Payment Agent - secure AI payment infrastructure"
        assert card.api_endpoint is not None
        assert card.api_endpoint.url == "https://api.sardis.sh/api/v2"
        assert card.api_endpoint.auth_required is True
        assert card.a2a_endpoint is not None
        assert card.a2a_endpoint.url == "https://api.sardis.sh/api/v2/a2a"

    def test_create_card_with_mcp(self):
        """Test creating card with MCP command."""
        card = create_sardis_agent_card(
            agent_id="agent_test",
            agent_name="Test Agent",
            api_base_url="https://api.sardis.sh",
            mcp_command="npx @sardis/mcp-server",
        )

        assert card.mcp_endpoint == "npx @sardis/mcp-server"

    def test_create_card_with_signing(self):
        """Test creating card with signing key."""
        card = create_sardis_agent_card(
            agent_id="agent_test",
            agent_name="Test Agent",
            api_base_url="https://api.sardis.sh",
            signing_key_id="key_123",
            public_key="base64_key",
        )

        assert card.signing_key_id == "key_123"
        assert card.public_key == "base64_key"



"""A2A client for outbound agent communication.

Provides methods for sending A2A messages to other agents:
- Payment requests
- Credential verification
- Checkout flows
"""

from __future__ import annotations

import logging
from dataclasses import dataclass, field
from datetime import datetime, timezone
from typing import Any, Dict, Optional, Protocol

from .messages import (
    A2AMessage,
    A2AMessageType,
    A2AMessageStatus,
    A2APaymentRequest,
    A2APaymentResponse,
    A2ACredentialRequest,
    A2ACredentialResponse,
)
from .discovery import AgentDiscoveryService, DiscoveredAgent

logger = logging.getLogger(__name__)


class A2AClientError(Exception):
    """Error during A2A client operations."""

    def __init__(
        self,
        message: str,
        code: str,
        recipient_id: str | None = None,
        details: Dict[str, Any] | None = None,
    ):
        super().__init__(message)
        self.code = code
        self.recipient_id = recipient_id
        self.details = details or {}


class HttpClient(Protocol):
    """Protocol for HTTP client operations."""

    async def post(
        self,
        url: str,
        json: Dict[str, Any],
        headers: Dict[str, str] | None = None,
    ) -> tuple[int, Dict[str, Any]]:
        """Make an HTTP POST request.

        Returns:
            Tuple of (status_code, json_response)
        """
        ...


class MessageSigner(Protocol):
    """Protocol for signing A2A messages."""

    def sign(self, message: A2AMessage) -> str:
        """Sign a message and return the signature."""
        ...

    def verify(self, message: A2AMessage, signature: str) -> bool:
        """Verify a message signature."""
        ...


@dataclass
class A2AClientConfig:
    """Configuration for A2A client."""

    # Client identity
    agent_id: str
    agent_name: str

    # Endpoints
    base_url: str  # Our base URL for callbacks

    # Signing
    signing_key_id: Optional[str] = None
    private_key: Optional[str] = None

    # Timeouts (seconds)
    request_timeout: int = 30
    message_ttl: int = 300  # 5 minutes

    # Retries
    max_retries: int = 3
    retry_delay: float = 1.0


class A2AClient:
    """
    Client for A2A inter-agent communication.

    Provides methods for:
    - Sending payment requests to other agents
    - Verifying credentials with other agents
    - General A2A message exchange

    Uses agent discovery to find agent endpoints.
    """

    def __init__(
        self,
        config: A2AClientConfig,
        http_client: HttpClient | None = None,
        discovery: AgentDiscoveryService | None = None,
        signer: MessageSigner | None = None,
    ) -> None:
        """
        Initialize the A2A client.

        Args:
            config: Client configuration
            http_client: HTTP client for requests
            discovery: Agent discovery service
            signer: Message signer for authentication
        """
        self._config = config
        self._http_client = http_client
        self._discovery = discovery or AgentDiscoveryService()
        self._signer = signer

    @property
    def agent_id(self) -> str:
        """Our agent ID."""
        return self._config.agent_id

    async def send_payment_request(
        self,
        recipient_url: str,
        amount_minor: int,
        token: str,
        chain: str,
        destination: str,
        purpose: str = "",
        reference: str | None = None,
        callback_url: str | None = None,
        metadata: Dict[str, Any] | None = None,
    ) -> A2APaymentResponse:
        """
        Send a payment request to another agent.

        Args:
            recipient_url: Base URL of the recipient agent
            amount_minor: Amount in minor units
            token: Token symbol (e.g., "USDC")
            chain: Blockchain network (e.g., "base")
            destination: Payment destination address
            purpose: Purpose of the payment
            reference: External reference (order ID, invoice, etc.)
            callback_url: URL for payment status callbacks
            metadata: Additional metadata

        Returns:
            A2APaymentResponse with result

        Raises:
            A2AClientError: If request fails
        """
        # Discover recipient
        recipient = await self._discovery.discover_agent(recipient_url)

        if not recipient.available:
            raise A2AClientError(
                f"Recipient agent not available: {recipient.last_error}",
                code="agent_unavailable",
                recipient_id=recipient.agent_id,
            )

        # Check capabilities
        if not recipient.supports_payment(token, chain):
            raise A2AClientError(
                f"Recipient does not support {token} on {chain}",
                code="unsupported_payment",
                recipient_id=recipient.agent_id,
            )

        # Build request
        request = A2APaymentRequest(
            sender_agent_id=self._config.agent_id,
            recipient_agent_id=recipient.agent_id,
            amount_minor=amount_minor,
            token=token,
            chain=chain,
            destination=destination,
            purpose=purpose,
            reference=reference,
            callback_url=callback_url or f"{self._config.base_url}/api/v2/a2a/callback",
            metadata=metadata or {},
        )

        # Convert to A2A message
        message = request.to_a2a_message()

        # Sign if signer available
        if self._signer:
            message.signature = self._signer.sign(message)

        # Send request
        response = await self._send_message(recipient, message)

        # Parse response
        if response.message_type == A2AMessageType.PAYMENT_RESPONSE:
            payload = response.payload
            return A2APaymentResponse(
                response_id=payload.get("response_id", response.message_id),
                request_id=request.request_id,
                sender_agent_id=response.sender_id,
                recipient_agent_id=response.recipient_id,
                success=payload.get("success", False),
                status=payload.get("status", "unknown"),
                tx_hash=payload.get("tx_hash"),
                chain=payload.get("chain"),
                block_number=payload.get("block_number"),
                error=payload.get("error") or response.error,
                error_code=payload.get("error_code") or response.error_code,
                metadata=payload.get("metadata", {}),
            )
        elif response.message_type == A2AMessageType.ERROR:
            return A2APaymentResponse(
                request_id=request.request_id,
                sender_agent_id=response.sender_id,
                recipient_agent_id=response.recipient_id,
                success=False,
                status="failed",
                error=response.error or "Unknown error",
                error_code=response.error_code or "unknown_error",
            )
        else:
            raise A2AClientError(
                f"Unexpected response type: {response.message_type}",
                code="unexpected_response",
                recipient_id=recipient.agent_id,
            )

    async def verify_credential(
        self,
        recipient_url: str,
        credential_type: str,
        credential_data: Dict[str, Any],
        verify_signature: bool = True,
        verify_expiration: bool = True,
        verify_chain: bool = True,
    ) -> A2ACredentialResponse:
        """
        Request credential verification from another agent.

        Args:
            recipient_url: Base URL of the verifying agent
            credential_type: Type of credential (mandate, identity, etc.)
            credential_data: The credential data to verify
            verify_signature: Whether to verify signatures
            verify_expiration: Whether to check expiration
            verify_chain: Whether to verify mandate chain

        Returns:
            A2ACredentialResponse with verification result

        Raises:
            A2AClientError: If request fails
        """
        # Discover recipient
        recipient = await self._discovery.discover_agent(recipient_url)

        if not recipient.available:
            raise A2AClientError(
                f"Recipient agent not available: {recipient.last_error}",
                code="agent_unavailable",
                recipient_id=recipient.agent_id,
            )

        # Build request
        request = A2ACredentialRequest(
            sender_agent_id=self._config.agent_id,
            recipient_agent_id=recipient.agent_id,
            credential_type=credential_type,
            credential_data=credential_data,
            verify_signature=verify_signature,
            verify_expiration=verify_expiration,
            verify_chain=verify_chain,
        )

        # Convert to A2A message
        message = request.to_a2a_message()

        # Sign if signer available
        if self._signer:
            message.signature = self._signer.sign(message)

        # Send request
        response = await self._send_message(recipient, message)

        # Parse response
        if response.message_type == A2AMessageType.CREDENTIAL_RESPONSE:
            payload = response.payload
            return A2ACredentialResponse(
                response_id=payload.get("response_id", response.message_id),
                request_id=request.request_id,
                sender_agent_id=response.sender_id,
                recipient_agent_id=response.recipient_id,
                valid=payload.get("valid", False),
                signature_valid=payload.get("signature_valid"),
                not_expired=payload.get("not_expired"),
                chain_valid=payload.get("chain_valid"),
                error=payload.get("error") or response.error,
                error_code=payload.get("error_code") or response.error_code,
                verification_details=payload.get("verification_details", {}),
            )
        elif response.message_type == A2AMessageType.ERROR:
            return A2ACredentialResponse(
                request_id=request.request_id,
                sender_agent_id=response.sender_id,
                recipient_agent_id=response.recipient_id,
                valid=False,
                error=response.error or "Unknown error",
                error_code=response.error_code or "unknown_error",
            )
        else:
            raise A2AClientError(
                f"Unexpected response type: {response.message_type}",
                code="unexpected_response",
                recipient_id=recipient.agent_id,
            )

    async def send_message(
        self,
        recipient_url: str,
        message: A2AMessage,
    ) -> A2AMessage:
        """
        Send a raw A2A message to another agent.

        Args:
            recipient_url: Base URL of the recipient agent
            message: Message to send

        Returns:
            Response message

        Raises:
            A2AClientError: If request fails
        """
        # Discover recipient
        recipient = await self._discovery.discover_agent(recipient_url)

        if not recipient.available:
            raise A2AClientError(
                f"Recipient agent not available: {recipient.last_error}",
                code="agent_unavailable",
                recipient_id=recipient.agent_id,
            )

        # Update sender ID
        message.sender_id = self._config.agent_id

        # Sign if signer available
        if self._signer and not message.signature:
            message.signature = self._signer.sign(message)

        return await self._send_message(recipient, message)

    async def _send_message(
        self,
        recipient: DiscoveredAgent,
        message: A2AMessage,
    ) -> A2AMessage:
        """Send a message to a discovered agent."""
        if self._http_client is None:
            raise A2AClientError(
                "HTTP client not configured",
                code="client_not_configured",
                recipient_id=recipient.agent_id,
            )

        # Determine A2A endpoint
        a2a_url = f"{recipient.agent_url}/api/v2/a2a/messages"
        if recipient.card and recipient.card.a2a_endpoint:
            a2a_url = recipient.card.a2a_endpoint.url

        logger.info(
            f"Sending A2A message: type={message.message_type.value}, "
            f"recipient={recipient.agent_id}, url={a2a_url}"
        )

        try:
            status, data = await self._http_client.post(
                a2a_url,
                json=message.to_dict(),
                headers={
                    "Content-Type": "application/json",
                    "X-Sardis-Agent-Id": self._config.agent_id,
                },
            )

            if status >= 400:
                raise A2AClientError(
                    f"HTTP {status} from {a2a_url}",
                    code=f"http_{status}",
                    recipient_id=recipient.agent_id,
                    details={"response": data},
                )

            # Parse response
            response = A2AMessage.from_dict(data)

            logger.info(
                f"Received A2A response: type={response.message_type.value}, "
                f"status={response.status.value}"
            )

            return response

        except A2AClientError:
            raise
        except Exception as e:
            logger.error(f"A2A message send failed: {e}")
            raise A2AClientError(
                f"Failed to send message: {e}",
                code="send_failed",
                recipient_id=recipient.agent_id,
                details={"error": str(e)},
            )


__all__ = [
    "A2AClientError",
    "HttpClient",
    "MessageSigner",
    "A2AClientConfig",
    "A2AClient",
]



"""Agent-to-Agent (A2A) protocol integration for Sardis.

A2A enables multi-agent communication with support for:
- Agent discovery via /.well-known/agent-card.json
- Structured message exchange (payment requests, credentials)
- Capability negotiation

This package provides Sardis's A2A implementation for agent interoperability.
"""

from .agent_card import (
    SardisAgentCard,
    AgentCapability,
    PaymentCapability,
    create_sardis_agent_card,
)
from .messages import (
    A2AMessageType,
    A2AMessage,
    A2APaymentRequest,
    A2APaymentResponse,
    A2ACredentialRequest,
    A2ACredentialResponse,
)
from .discovery import (
    AgentDiscoveryService,
    DiscoveredAgent,
)
from .client import (
    A2AClient,
    A2AClientError,
)

__all__ = [
    # Agent Card
    "SardisAgentCard",
    "AgentCapability",
    "PaymentCapability",
    "create_sardis_agent_card",
    # Messages
    "A2AMessageType",
    "A2AMessage",
    "A2APaymentRequest",
    "A2APaymentResponse",
    "A2ACredentialRequest",
    "A2ACredentialResponse",
    # Discovery
    "AgentDiscoveryService",
    "DiscoveredAgent",
    # Client
    "A2AClient",
    "A2AClientError",
]



"""A2A message types for inter-agent communication.

Defines structured message formats for:
- Payment requests/responses
- Credential verification
- Checkout flows
"""

from __future__ import annotations

import uuid
from dataclasses import dataclass, field
from datetime import datetime, timezone
from enum import Enum
from typing import Any, Dict, List, Optional


class A2AMessageType(str, Enum):
    """Types of A2A messages."""

    # Payment messages
    PAYMENT_REQUEST = "payment_request"
    PAYMENT_RESPONSE = "payment_response"

    # Credential messages
    CREDENTIAL_REQUEST = "credential_request"
    CREDENTIAL_RESPONSE = "credential_response"

    # Checkout messages (UCP)
    CHECKOUT_INITIATE = "checkout_initiate"
    CHECKOUT_COMPLETE = "checkout_complete"

    # Status/Error
    ACK = "ack"
    ERROR = "error"


class A2AMessageStatus(str, Enum):
    """Status of an A2A message."""

    PENDING = "pending"
    RECEIVED = "received"
    PROCESSING = "processing"
    COMPLETED = "completed"
    FAILED = "failed"


@dataclass(slots=True)
class A2AMessage:
    """
    Base A2A message structure.

    All A2A messages follow this structure with type-specific payloads.
    """

    message_id: str = field(default_factory=lambda: str(uuid.uuid4()))
    message_type: A2AMessageType = A2AMessageType.ACK
    sender_id: str = ""
    recipient_id: str = ""

    # Timing
    timestamp: datetime = field(default_factory=lambda: datetime.now(timezone.utc))
    expires_at: Optional[datetime] = None

    # Request correlation
    correlation_id: Optional[str] = None  # Links request/response pairs
    in_reply_to: Optional[str] = None  # Message this is responding to

    # Payload
    payload: Dict[str, Any] = field(default_factory=dict)

    # Signature
    signature: Optional[str] = None
    signature_algorithm: str = "Ed25519"

    # Status
    status: A2AMessageStatus = A2AMessageStatus.PENDING

    # Error (if failed)
    error: Optional[str] = None
    error_code: Optional[str] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for serialization."""
        return {
            "message_id": self.message_id,
            "message_type": self.message_type.value,
            "sender_id": self.sender_id,
            "recipient_id": self.recipient_id,
            "timestamp": self.timestamp.isoformat(),
            "expires_at": self.expires_at.isoformat() if self.expires_at else None,
            "correlation_id": self.correlation_id,
            "in_reply_to": self.in_reply_to,
            "payload": self.payload,
            "signature": self.signature,
            "signature_algorithm": self.signature_algorithm,
            "status": self.status.value,
            "error": self.error,
            "error_code": self.error_code,
        }

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "A2AMessage":
        """Create from dictionary."""
        return cls(
            message_id=data.get("message_id", str(uuid.uuid4())),
            message_type=A2AMessageType(data.get("message_type", "ack")),
            sender_id=data.get("sender_id", ""),
            recipient_id=data.get("recipient_id", ""),
            timestamp=datetime.fromisoformat(data["timestamp"]) if "timestamp" in data else datetime.now(timezone.utc),
            expires_at=datetime.fromisoformat(data["expires_at"]) if data.get("expires_at") else None,
            correlation_id=data.get("correlation_id"),
            in_reply_to=data.get("in_reply_to"),
            payload=data.get("payload", {}),
            signature=data.get("signature"),
            signature_algorithm=data.get("signature_algorithm", "Ed25519"),
            status=A2AMessageStatus(data.get("status", "pending")),
            error=data.get("error"),
            error_code=data.get("error_code"),
        )


# ============ Payment Messages ============


@dataclass(slots=True)
class A2APaymentRequest:
    """
    Request to execute a payment.

    Sent from one agent to another to request payment execution.
    """

    # Request metadata
    request_id: str = field(default_factory=lambda: str(uuid.uuid4()))
    sender_agent_id: str = ""
    recipient_agent_id: str = ""

    # Payment details
    amount_minor: int = 0
    currency: str = "USD"
    token: str = "USDC"
    chain: str = "base"
    destination: str = ""

    # Purpose/context
    purpose: str = ""
    reference: Optional[str] = None  # External reference (invoice, order ID, etc.)
    memo: Optional[str] = None

    # Mandate information
    mandate_id: Optional[str] = None
    audit_hash: Optional[str] = None

    # Timing
    created_at: datetime = field(default_factory=lambda: datetime.now(timezone.utc))
    expires_at: Optional[datetime] = None

    # Callback
    callback_url: Optional[str] = None

    # Metadata
    metadata: Dict[str, Any] = field(default_factory=dict)

    def to_a2a_message(self) -> A2AMessage:
        """Convert to A2A message format."""
        return A2AMessage(
            message_type=A2AMessageType.PAYMENT_REQUEST,
            sender_id=self.sender_agent_id,
            recipient_id=self.recipient_agent_id,
            correlation_id=self.request_id,
            payload={
                "request_id": self.request_id,
                "amount_minor": self.amount_minor,
                "currency": self.currency,
                "token": self.token,
                "chain": self.chain,
                "destination": self.destination,
                "purpose": self.purpose,
                "reference": self.reference,
                "memo": self.memo,
                "mandate_id": self.mandate_id,
                "audit_hash": self.audit_hash,
                "callback_url": self.callback_url,
                "metadata": self.metadata,
            },
            expires_at=self.expires_at,
        )

    @classmethod
    def from_a2a_message(cls, message: A2AMessage) -> "A2APaymentRequest":
        """Create from A2A message."""
        payload = message.payload
        return cls(
            request_id=payload.get("request_id", message.correlation_id or str(uuid.uuid4())),
            sender_agent_id=message.sender_id,
            recipient_agent_id=message.recipient_id,
            amount_minor=payload.get("amount_minor", 0),
            currency=payload.get("currency", "USD"),
            token=payload.get("token", "USDC"),
            chain=payload.get("chain", "base"),
            destination=payload.get("destination", ""),
            purpose=payload.get("purpose", ""),
            reference=payload.get("reference"),
            memo=payload.get("memo"),
            mandate_id=payload.get("mandate_id"),
            audit_hash=payload.get("audit_hash"),
            created_at=message.timestamp,
            expires_at=message.expires_at,
            callback_url=payload.get("callback_url"),
            metadata=payload.get("metadata", {}),
        )


@dataclass(slots=True)
class A2APaymentResponse:
    """
    Response to a payment request.

    Sent after payment execution (success or failure).
    """

    # Response metadata
    response_id: str = field(default_factory=lambda: str(uuid.uuid4()))
    request_id: str = ""  # ID of the original request
    sender_agent_id: str = ""
    recipient_agent_id: str = ""

    # Result
    success: bool = False
    status: str = "pending"  # pending, submitted, confirmed, failed

    # Transaction details (if successful)
    tx_hash: Optional[str] = None
    chain: Optional[str] = None
    block_number: Optional[int] = None

    # Error details (if failed)
    error: Optional[str] = None
    error_code: Optional[str] = None

    # Timing
    created_at: datetime = field(default_factory=lambda: datetime.now(timezone.utc))

    # Metadata
    metadata: Dict[str, Any] = field(default_factory=dict)

    def to_a2a_message(self) -> A2AMessage:
        """Convert to A2A message format."""
        return A2AMessage(
            message_type=A2AMessageType.PAYMENT_RESPONSE,
            sender_id=self.sender_agent_id,
            recipient_id=self.recipient_agent_id,
            correlation_id=self.response_id,
            in_reply_to=self.request_id,
            payload={
                "response_id": self.response_id,
                "request_id": self.request_id,
                "success": self.success,
                "status": self.status,
                "tx_hash": self.tx_hash,
                "chain": self.chain,
                "block_number": self.block_number,
                "error": self.error,
                "error_code": self.error_code,
                "metadata": self.metadata,
            },
            status=A2AMessageStatus.COMPLETED if self.success else A2AMessageStatus.FAILED,
            error=self.error,
            error_code=self.error_code,
        )


# ============ Credential Messages ============


@dataclass(slots=True)
class A2ACredentialRequest:
    """
    Request to verify credentials or mandates.

    Sent to request verification of a credential chain or mandate.
    """

    request_id: str = field(default_factory=lambda: str(uuid.uuid4()))
    sender_agent_id: str = ""
    recipient_agent_id: str = ""

    # Credential to verify
    credential_type: str = "mandate"  # mandate, identity, authorization
    credential_data: Dict[str, Any] = field(default_factory=dict)

    # What to verify
    verify_signature: bool = True
    verify_expiration: bool = True
    verify_chain: bool = True  # For mandate chains

    # Timing
    created_at: datetime = field(default_factory=lambda: datetime.now(timezone.utc))

    def to_a2a_message(self) -> A2AMessage:
        """Convert to A2A message format."""
        return A2AMessage(
            message_type=A2AMessageType.CREDENTIAL_REQUEST,
            sender_id=self.sender_agent_id,
            recipient_id=self.recipient_agent_id,
            correlation_id=self.request_id,
            payload={
                "request_id": self.request_id,
                "credential_type": self.credential_type,
                "credential_data": self.credential_data,
                "verify_signature": self.verify_signature,
                "verify_expiration": self.verify_expiration,
                "verify_chain": self.verify_chain,
            },
        )


@dataclass(slots=True)
class A2ACredentialResponse:
    """
    Response to a credential verification request.
    """

    response_id: str = field(default_factory=lambda: str(uuid.uuid4()))
    request_id: str = ""
    sender_agent_id: str = ""
    recipient_agent_id: str = ""

    # Verification result
    valid: bool = False
    verified_at: datetime = field(default_factory=lambda: datetime.now(timezone.utc))

    # Verification details
    signature_valid: Optional[bool] = None
    not_expired: Optional[bool] = None
    chain_valid: Optional[bool] = None

    # Error (if failed)
    error: Optional[str] = None
    error_code: Optional[str] = None

    # Additional info from verification
    verification_details: Dict[str, Any] = field(default_factory=dict)

    def to_a2a_message(self) -> A2AMessage:
        """Convert to A2A message format."""
        return A2AMessage(
            message_type=A2AMessageType.CREDENTIAL_RESPONSE,
            sender_id=self.sender_agent_id,
            recipient_id=self.recipient_agent_id,
            correlation_id=self.response_id,
            in_reply_to=self.request_id,
            payload={
                "response_id": self.response_id,
                "request_id": self.request_id,
                "valid": self.valid,
                "verified_at": self.verified_at.isoformat(),
                "signature_valid": self.signature_valid,
                "not_expired": self.not_expired,
                "chain_valid": self.chain_valid,
                "error": self.error,
                "error_code": self.error_code,
                "verification_details": self.verification_details,
            },
            status=A2AMessageStatus.COMPLETED if self.valid else A2AMessageStatus.FAILED,
        )


__all__ = [
    "A2AMessageType",
    "A2AMessageStatus",
    "A2AMessage",
    "A2APaymentRequest",
    "A2APaymentResponse",
    "A2ACredentialRequest",
    "A2ACredentialResponse",
]



"""Tests for A2A client."""

from datetime import datetime, timezone
from typing import Any, Dict

import pytest

from sardis_a2a.agent_card import (
    AgentCapability,
    PaymentCapability,
    SardisAgentCard,
    ServiceEndpoint,
)
from sardis_a2a.messages import (
    A2AMessage,
    A2AMessageType,
    A2AMessageStatus,
)
from sardis_a2a.discovery import (
    DiscoveredAgent,
    AgentDiscoveryService,
)
from sardis_a2a.client import (
    A2AClient,
    A2AClientConfig,
    A2AClientError,
)


class MockHttpClient:
    """Mock HTTP client for testing."""

    def __init__(self):
        self.get_responses: Dict[str, tuple[int, Dict[str, Any]]] = {}
        self.post_responses: Dict[str, tuple[int, Dict[str, Any]]] = {}
        self.post_calls: list[tuple[str, Dict[str, Any]]] = []

    def add_get_response(self, url: str, status: int, data: Dict[str, Any]):
        """Add a mock GET response."""
        self.get_responses[url] = (status, data)

    def add_post_response(self, url: str, status: int, data: Dict[str, Any]):
        """Add a mock POST response."""
        self.post_responses[url] = (status, data)

    async def get(
        self, url: str, headers: Dict[str, str] | None = None
    ) -> tuple[int, Dict[str, Any]]:
        """Mock GET request."""
        if url in self.get_responses:
            return self.get_responses[url]
        return 404, {"error": "Not found"}

    async def post(
        self,
        url: str,
        json: Dict[str, Any],
        headers: Dict[str, str] | None = None,
    ) -> tuple[int, Dict[str, Any]]:
        """Mock POST request."""
        self.post_calls.append((url, json))
        if url in self.post_responses:
            return self.post_responses[url]
        return 404, {"error": "Not found"}


class MockSigner:
    """Mock message signer for testing."""

    def sign(self, message: A2AMessage) -> str:
        """Return a mock signature."""
        return f"mock_sig_{message.message_id[:8]}"

    def verify(self, message: A2AMessage, signature: str) -> bool:
        """Verify signature matches our mock format."""
        return signature.startswith("mock_sig_")


class TestA2AClientError:
    """Tests for A2AClientError."""

    def test_create_error(self):
        """Test creating an A2A client error."""
        error = A2AClientError(
            message="Connection failed",
            code="connection_error",
            recipient_id="agent_123",
            details={"host": "example.com"},
        )

        assert str(error) == "Connection failed"
        assert error.code == "connection_error"
        assert error.recipient_id == "agent_123"
        assert error.details["host"] == "example.com"

    def test_error_without_details(self):
        """Test error without optional fields."""
        error = A2AClientError("Error", code="generic")

        assert error.recipient_id is None
        assert error.details == {}


class TestA2AClientConfig:
    """Tests for A2AClientConfig."""

    def test_create_config(self):
        """Test creating client config."""
        config = A2AClientConfig(
            agent_id="agent_sender",
            agent_name="Sender Agent",
            base_url="https://sender.example.com",
        )

        assert config.agent_id == "agent_sender"
        assert config.agent_name == "Sender Agent"
        assert config.base_url == "https://sender.example.com"

    def test_default_values(self):
        """Test default config values."""
        config = A2AClientConfig(
            agent_id="agent_1",
            agent_name="Agent",
            base_url="https://example.com",
        )

        assert config.request_timeout == 30
        assert config.message_ttl == 300
        assert config.max_retries == 3
        assert config.retry_delay == 1.0

    def test_custom_values(self):
        """Test custom config values."""
        config = A2AClientConfig(
            agent_id="agent_1",
            agent_name="Agent",
            base_url="https://example.com",
            signing_key_id="key_123",
            private_key="private_key_data",
            request_timeout=60,
            message_ttl=600,
        )

        assert config.signing_key_id == "key_123"
        assert config.request_timeout == 60
        assert config.message_ttl == 600


class TestA2AClient:
    """Tests for A2AClient."""

    @pytest.fixture
    def http_client(self):
        """Create a mock HTTP client."""
        return MockHttpClient()

    @pytest.fixture
    def config(self):
        """Create a client config."""
        return A2AClientConfig(
            agent_id="agent_sender",
            agent_name="Sender Agent",
            base_url="https://sender.example.com",
        )

    @pytest.fixture
    def recipient_card(self):
        """Create a sample recipient agent card."""
        return SardisAgentCard(
            agent_id="agent_recipient",
            agent_name="Recipient Agent",
            capabilities=[
                AgentCapability.PAYMENT_EXECUTE,
                AgentCapability.PAYMENT_VERIFY,
            ],
            payment_capability=PaymentCapability(
                supported_tokens=["USDC", "USDT"],
                supported_chains=["base", "polygon"],
            ),
            a2a_endpoint=ServiceEndpoint(
                url="https://recipient.example.com/api/v2/a2a/messages",
            ),
        )

    @pytest.fixture
    def agent_card_data(self, recipient_card):
        """Agent card JSON data."""
        return recipient_card.to_dict()

    @pytest.fixture
    def discovery(self, http_client, agent_card_data):
        """Create discovery service with mock responses."""
        http_client.add_get_response(
            "https://recipient.example.com/.well-known/agent-card.json",
            200,
            agent_card_data,
        )
        return AgentDiscoveryService(http_client=http_client)

    @pytest.fixture
    def client(self, config, http_client, discovery):
        """Create an A2A client."""
        return A2AClient(
            config=config,
            http_client=http_client,
            discovery=discovery,
        )

    def test_create_client(self, client, config):
        """Test creating a client."""
        assert client.agent_id == "agent_sender"

    @pytest.mark.asyncio
    async def test_send_payment_request_success(self, client, http_client):
        """Test successful payment request."""
        # Mock successful payment response
        http_client.add_post_response(
            "https://recipient.example.com/api/v2/a2a/messages",
            200,
            {
                "message_id": "resp_123",
                "message_type": "payment_response",
                "sender_id": "agent_recipient",
                "recipient_id": "agent_sender",
                "timestamp": datetime.now(timezone.utc).isoformat(),
                "status": "completed",
                "payload": {
                    "response_id": "resp_123",
                    "request_id": "req_123",
                    "success": True,
                    "status": "confirmed",
                    "tx_hash": "0xabcdef123456",
                    "chain": "base",
                    "block_number": 12345,
                },
            },
        )

        response = await client.send_payment_request(
            recipient_url="https://recipient.example.com",
            amount_minor=5000,
            token="USDC",
            chain="base",
            destination="0x1234567890abcdef1234567890abcdef12345678",
            purpose="Test payment",
        )

        assert response.success is True
        assert response.status == "confirmed"
        assert response.tx_hash == "0xabcdef123456"
        assert response.chain == "base"
        assert response.block_number == 12345

    @pytest.mark.asyncio
    async def test_send_payment_request_failure(self, client, http_client):
        """Test failed payment request."""
        http_client.add_post_response(
            "https://recipient.example.com/api/v2/a2a/messages",
            200,
            {
                "message_id": "resp_123",
                "message_type": "payment_response",
                "sender_id": "agent_recipient",
                "recipient_id": "agent_sender",
                "timestamp": datetime.now(timezone.utc).isoformat(),
                "status": "failed",
                "payload": {
                    "success": False,
                    "status": "failed",
                    "error": "Insufficient balance",
                    "error_code": "insufficient_balance",
                },
            },
        )

        response = await client.send_payment_request(
            recipient_url="https://recipient.example.com",
            amount_minor=1_000_000_00,  # $1M
            token="USDC",
            chain="base",
            destination="0x1234567890abcdef1234567890abcdef12345678",
        )

        assert response.success is False
        assert response.error == "Insufficient balance"
        assert response.error_code == "insufficient_balance"

    @pytest.mark.asyncio
    async def test_send_payment_request_unsupported_payment(self, client):
        """Test payment request with unsupported token/chain."""
        with pytest.raises(A2AClientError) as exc_info:
            await client.send_payment_request(
                recipient_url="https://recipient.example.com",
                amount_minor=5000,
                token="UNKNOWN",  # Not supported
                chain="base",
                destination="0x1234",
            )

        assert exc_info.value.code == "unsupported_payment"

    @pytest.mark.asyncio
    async def test_send_payment_request_agent_unavailable(self, config, http_client):
        """Test payment request to unavailable agent."""
        # Discovery will fail - no mock response added
        discovery = AgentDiscoveryService(http_client=http_client)
        client = A2AClient(config=config, http_client=http_client, discovery=discovery)

        with pytest.raises(A2AClientError) as exc_info:
            await client.send_payment_request(
                recipient_url="https://unavailable.example.com",
                amount_minor=5000,
                token="USDC",
                chain="base",
                destination="0x1234",
            )

        assert exc_info.value.code == "agent_unavailable"

    @pytest.mark.asyncio
    async def test_send_payment_request_with_metadata(self, client, http_client):
        """Test payment request with metadata."""
        http_client.add_post_response(
            "https://recipient.example.com/api/v2/a2a/messages",
            200,
            {
                "message_id": "resp_123",
                "message_type": "payment_response",
                "sender_id": "agent_recipient",
                "recipient_id": "agent_sender",
                "timestamp": datetime.now(timezone.utc).isoformat(),
                "status": "completed",
                "payload": {
                    "success": True,
                    "status": "confirmed",
                    "tx_hash": "0xabc",
                },
            },
        )

        await client.send_payment_request(
            recipient_url="https://recipient.example.com",
            amount_minor=5000,
            token="USDC",
            chain="base",
            destination="0x1234567890abcdef1234567890abcdef12345678",
            reference="order_123",
            metadata={"order_id": "123", "customer_id": "cust_456"},
        )

        # Check that metadata was included in request
        assert len(http_client.post_calls) == 1
        _, sent_data = http_client.post_calls[0]
        assert sent_data["payload"]["reference"] == "order_123"
        assert sent_data["payload"]["metadata"]["order_id"] == "123"

    @pytest.mark.asyncio
    async def test_verify_credential_success(self, client, http_client):
        """Test successful credential verification."""
        http_client.add_post_response(
            "https://recipient.example.com/api/v2/a2a/messages",
            200,
            {
                "message_id": "resp_123",
                "message_type": "credential_response",
                "sender_id": "agent_recipient",
                "recipient_id": "agent_sender",
                "timestamp": datetime.now(timezone.utc).isoformat(),
                "status": "completed",
                "payload": {
                    "valid": True,
                    "signature_valid": True,
                    "not_expired": True,
                    "chain_valid": True,
                    "verification_details": {"verified_at": "2024-01-15T10:00:00Z"},
                },
            },
        )

        response = await client.verify_credential(
            recipient_url="https://recipient.example.com",
            credential_type="mandate",
            credential_data={"mandate_id": "mand_123"},
        )

        assert response.valid is True
        assert response.signature_valid is True
        assert response.not_expired is True
        assert response.chain_valid is True

    @pytest.mark.asyncio
    async def test_verify_credential_invalid(self, client, http_client):
        """Test invalid credential verification."""
        http_client.add_post_response(
            "https://recipient.example.com/api/v2/a2a/messages",
            200,
            {
                "message_id": "resp_123",
                "message_type": "credential_response",
                "sender_id": "agent_recipient",
                "recipient_id": "agent_sender",
                "timestamp": datetime.now(timezone.utc).isoformat(),
                "status": "failed",
                "payload": {
                    "valid": False,
                    "signature_valid": False,
                    "error": "Invalid signature",
                    "error_code": "invalid_signature",
                },
            },
        )

        response = await client.verify_credential(
            recipient_url="https://recipient.example.com",
            credential_type="mandate",
            credential_data={"mandate_id": "mand_invalid"},
        )

        assert response.valid is False
        assert response.signature_valid is False
        assert response.error == "Invalid signature"

    @pytest.mark.asyncio
    async def test_send_message_raw(self, client, http_client):
        """Test sending raw A2A message."""
        http_client.add_post_response(
            "https://recipient.example.com/api/v2/a2a/messages",
            200,
            {
                "message_id": "resp_123",
                "message_type": "ack",
                "sender_id": "agent_recipient",
                "recipient_id": "agent_sender",
                "timestamp": datetime.now(timezone.utc).isoformat(),
                "status": "received",
                "payload": {},
            },
        )

        message = A2AMessage(
            message_type=A2AMessageType.ACK,
            recipient_id="agent_recipient",
            payload={"data": "test"},
        )

        response = await client.send_message(
            recipient_url="https://recipient.example.com",
            message=message,
        )

        assert response.message_type == A2AMessageType.ACK
        assert response.status == A2AMessageStatus.RECEIVED

    @pytest.mark.asyncio
    async def test_send_message_no_http_client(self, config, discovery):
        """Test sending message without HTTP client."""
        client = A2AClient(
            config=config,
            http_client=None,  # No HTTP client
            discovery=discovery,
        )

        # Pre-register agent so discovery doesn't need HTTP
        discovery.register_agent(
            agent_id="agent_recipient",
            agent_name="Recipient",
            agent_url="https://recipient.example.com",
            card=SardisAgentCard(
                agent_id="agent_recipient",
                agent_name="Recipient",
            ),
        )

        with pytest.raises(A2AClientError) as exc_info:
            await client.send_message(
                recipient_url="https://recipient.example.com",
                message=A2AMessage(),
            )

        assert exc_info.value.code == "client_not_configured"

    @pytest.mark.asyncio
    async def test_send_message_http_error(self, client, http_client):
        """Test handling HTTP error response."""
        http_client.add_post_response(
            "https://recipient.example.com/api/v2/a2a/messages",
            500,
            {"error": "Internal server error"},
        )

        with pytest.raises(A2AClientError) as exc_info:
            await client.send_message(
                recipient_url="https://recipient.example.com",
                message=A2AMessage(),
            )

        assert exc_info.value.code == "http_500"

    @pytest.mark.asyncio
    async def test_client_with_signer(self, config, http_client, discovery):
        """Test client with message signer."""
        signer = MockSigner()
        client = A2AClient(
            config=config,
            http_client=http_client,
            discovery=discovery,
            signer=signer,
        )

        http_client.add_post_response(
            "https://recipient.example.com/api/v2/a2a/messages",
            200,
            {
                "message_id": "resp_123",
                "message_type": "ack",
                "sender_id": "agent_recipient",
                "recipient_id": "agent_sender",
                "timestamp": datetime.now(timezone.utc).isoformat(),
                "status": "received",
                "payload": {},
            },
        )

        await client.send_message(
            recipient_url="https://recipient.example.com",
            message=A2AMessage(),
        )

        # Check that message was signed
        _, sent_data = http_client.post_calls[0]
        assert sent_data["signature"] is not None
        assert sent_data["signature"].startswith("mock_sig_")

    @pytest.mark.asyncio
    async def test_error_response_handling(self, client, http_client):
        """Test handling error type response."""
        http_client.add_post_response(
            "https://recipient.example.com/api/v2/a2a/messages",
            200,
            {
                "message_id": "resp_123",
                "message_type": "error",
                "sender_id": "agent_recipient",
                "recipient_id": "agent_sender",
                "timestamp": datetime.now(timezone.utc).isoformat(),
                "status": "failed",
                "error": "Rate limit exceeded",
                "error_code": "rate_limited",
                "payload": {},
            },
        )

        response = await client.send_payment_request(
            recipient_url="https://recipient.example.com",
            amount_minor=5000,
            token="USDC",
            chain="base",
            destination="0x1234567890abcdef1234567890abcdef12345678",
        )

        assert response.success is False
        assert response.error == "Rate limit exceeded"
        assert response.error_code == "rate_limited"



"""Tests for A2A agent discovery service."""

from datetime import datetime, timezone, timedelta
from typing import Any, Dict

import pytest

from sardis_a2a.agent_card import (
    AgentCapability,
    PaymentCapability,
    SardisAgentCard,
)
from sardis_a2a.discovery import (
    DiscoveredAgent,
    AgentDiscoveryService,
)


class MockHttpClient:
    """Mock HTTP client for testing."""

    def __init__(self):
        self.responses: Dict[str, tuple[int, Dict[str, Any]]] = {}

    def add_response(self, url: str, status: int, data: Dict[str, Any]):
        """Add a mock response for a URL."""
        self.responses[url] = (status, data)

    async def get(
        self, url: str, headers: Dict[str, str] | None = None
    ) -> tuple[int, Dict[str, Any]]:
        """Mock GET request."""
        if url in self.responses:
            return self.responses[url]
        return 404, {"error": "Not found"}


class TestDiscoveredAgent:
    """Tests for DiscoveredAgent."""

    @pytest.fixture
    def agent(self):
        """Create a sample discovered agent."""
        card = SardisAgentCard(
            agent_id="agent_123",
            agent_name="Test Agent",
            capabilities=[
                AgentCapability.PAYMENT_EXECUTE,
                AgentCapability.CHECKOUT_CREATE,
            ],
            payment_capability=PaymentCapability(
                supported_tokens=["USDC", "USDT"],
                supported_chains=["base", "polygon"],
            ),
        )

        return DiscoveredAgent(
            agent_id="agent_123",
            agent_name="Test Agent",
            agent_url="https://agent.example.com",
            card=card,
            cache_ttl_seconds=3600,
        )

    def test_create_discovered_agent(self, agent):
        """Test creating a discovered agent."""
        assert agent.agent_id == "agent_123"
        assert agent.agent_name == "Test Agent"
        assert agent.agent_url == "https://agent.example.com"
        assert agent.available is True
        assert agent.card is not None

    def test_cache_valid_fresh(self, agent):
        """Test cache validity for fresh agent."""
        assert agent.is_cache_valid() is True

    def test_cache_invalid_expired(self, agent):
        """Test cache validity for expired agent."""
        agent.last_verified_at = datetime.now(timezone.utc) - timedelta(hours=2)
        assert agent.is_cache_valid() is False

    def test_supports_capability(self, agent):
        """Test capability checking."""
        assert agent.supports_capability(AgentCapability.PAYMENT_EXECUTE) is True
        assert agent.supports_capability(AgentCapability.CHECKOUT_CREATE) is True
        assert agent.supports_capability(AgentCapability.PAYMENT_REFUND) is False

    def test_supports_capability_no_card(self):
        """Test capability checking without card."""
        agent = DiscoveredAgent(
            agent_id="agent_1",
            agent_name="Agent",
            agent_url="https://example.com",
            card=None,
        )

        assert agent.supports_capability(AgentCapability.PAYMENT_EXECUTE) is False

    def test_supports_payment(self, agent):
        """Test payment support checking."""
        assert agent.supports_payment("USDC", "base") is True
        assert agent.supports_payment("USDT", "polygon") is True
        assert agent.supports_payment("USDC", "unknown") is False
        assert agent.supports_payment("UNKNOWN", "base") is False

    def test_supports_payment_no_card(self):
        """Test payment support without card."""
        agent = DiscoveredAgent(
            agent_id="agent_1",
            agent_name="Agent",
            agent_url="https://example.com",
            card=None,
        )

        assert agent.supports_payment("USDC", "base") is False


class TestAgentDiscoveryService:
    """Tests for AgentDiscoveryService."""

    @pytest.fixture
    def http_client(self):
        """Create a mock HTTP client."""
        return MockHttpClient()

    @pytest.fixture
    def discovery(self, http_client):
        """Create a discovery service."""
        return AgentDiscoveryService(
            http_client=http_client,
            cache_ttl_seconds=3600,
        )

    @pytest.fixture
    def agent_card_data(self):
        """Sample agent card JSON data."""
        return {
            "agent_id": "agent_test",
            "name": "Test Agent",
            "version": "1.0.0",
            "description": "A test agent",
            "operator": {
                "name": "Test Operator",
                "url": "https://test.com",
            },
            "capabilities": [
                "payment.execute",
                "payment.verify",
                "checkout.create",
            ],
            "payment": {
                "supported_tokens": ["USDC", "USDT"],
                "supported_chains": ["base", "polygon"],
                "min_amount_minor": 100,
                "max_amount_minor": 100_000_00,
                "ap2_compliant": True,
            },
        }

    @pytest.mark.asyncio
    async def test_discover_agent_success(self, discovery, http_client, agent_card_data):
        """Test successful agent discovery."""
        http_client.add_response(
            "https://agent.example.com/.well-known/agent-card.json",
            200,
            agent_card_data,
        )

        agent = await discovery.discover_agent("https://agent.example.com")

        assert agent.agent_id == "agent_test"
        assert agent.agent_name == "Test Agent"
        assert agent.available is True
        assert agent.card is not None
        assert agent.card.agent_version == "1.0.0"
        assert len(agent.card.capabilities) == 3

    @pytest.mark.asyncio
    async def test_discover_agent_with_trailing_slash(self, discovery, http_client, agent_card_data):
        """Test discovery with trailing slash in URL."""
        http_client.add_response(
            "https://agent.example.com/.well-known/agent-card.json",
            200,
            agent_card_data,
        )

        agent = await discovery.discover_agent("https://agent.example.com/")

        assert agent.agent_id == "agent_test"

    @pytest.mark.asyncio
    async def test_discover_agent_cached(self, discovery, http_client, agent_card_data):
        """Test that discovered agents are cached."""
        http_client.add_response(
            "https://agent.example.com/.well-known/agent-card.json",
            200,
            agent_card_data,
        )

        # First discovery
        agent1 = await discovery.discover_agent("https://agent.example.com")

        # Second discovery should use cache
        agent2 = await discovery.discover_agent("https://agent.example.com")

        assert agent1 is agent2

    @pytest.mark.asyncio
    async def test_discover_agent_force_refresh(self, discovery, http_client, agent_card_data):
        """Test force refresh bypasses cache."""
        http_client.add_response(
            "https://agent.example.com/.well-known/agent-card.json",
            200,
            agent_card_data,
        )

        agent1 = await discovery.discover_agent("https://agent.example.com")

        # Modify response
        agent_card_data["name"] = "Updated Agent"
        http_client.add_response(
            "https://agent.example.com/.well-known/agent-card.json",
            200,
            agent_card_data,
        )

        # Force refresh
        agent2 = await discovery.discover_agent(
            "https://agent.example.com", force_refresh=True
        )

        assert agent2.agent_name == "Updated Agent"

    @pytest.mark.asyncio
    async def test_discover_agent_http_error(self, discovery, http_client):
        """Test discovery with HTTP error."""
        http_client.add_response(
            "https://agent.example.com/.well-known/agent-card.json",
            500,
            {"error": "Internal server error"},
        )

        agent = await discovery.discover_agent("https://agent.example.com")

        assert agent.available is False
        assert agent.last_error is not None
        assert "500" in agent.last_error

    @pytest.mark.asyncio
    async def test_discover_agent_no_http_client(self):
        """Test discovery without HTTP client."""
        discovery = AgentDiscoveryService(http_client=None)

        agent = await discovery.discover_agent("https://agent.example.com")

        assert agent.available is False
        assert "HTTP client not configured" in agent.last_error

    def test_register_agent(self, discovery):
        """Test manual agent registration."""
        card = SardisAgentCard(
            agent_id="agent_manual",
            agent_name="Manual Agent",
        )

        agent = discovery.register_agent(
            agent_id="agent_manual",
            agent_name="Manual Agent",
            agent_url="https://manual.example.com",
            card=card,
        )

        assert agent.agent_id == "agent_manual"
        assert agent.agent_name == "Manual Agent"
        assert agent.available is True
        assert agent.card is card

    def test_register_agent_normalizes_url(self, discovery):
        """Test that registration normalizes URL."""
        discovery.register_agent(
            agent_id="agent_1",
            agent_name="Agent",
            agent_url="https://example.com/",
        )

        # Should be found without trailing slash
        agent = discovery.get_agent("https://example.com")
        assert agent is not None

    def test_get_agent(self, discovery):
        """Test getting agent by URL."""
        discovery.register_agent(
            agent_id="agent_1",
            agent_name="Agent",
            agent_url="https://agent.example.com",
        )

        agent = discovery.get_agent("https://agent.example.com")

        assert agent is not None
        assert agent.agent_id == "agent_1"

    def test_get_agent_not_found(self, discovery):
        """Test getting non-existent agent."""
        agent = discovery.get_agent("https://nonexistent.example.com")

        assert agent is None

    def test_get_agent_by_id(self, discovery):
        """Test getting agent by ID."""
        discovery.register_agent(
            agent_id="agent_test_id",
            agent_name="Test Agent",
            agent_url="https://agent.example.com",
        )

        agent = discovery.get_agent_by_id("agent_test_id")

        assert agent is not None
        assert agent.agent_url == "https://agent.example.com"

    def test_get_agent_by_id_not_found(self, discovery):
        """Test getting non-existent agent by ID."""
        agent = discovery.get_agent_by_id("nonexistent_id")

        assert agent is None

    def test_list_agents_all(self, discovery):
        """Test listing all agents."""
        discovery.register_agent("agent_1", "Agent 1", "https://agent1.example.com")
        discovery.register_agent("agent_2", "Agent 2", "https://agent2.example.com")
        discovery.register_agent("agent_3", "Agent 3", "https://agent3.example.com")

        agents = discovery.list_agents()

        assert len(agents) == 3

    def test_list_agents_available_only(self, discovery):
        """Test listing only available agents."""
        discovery.register_agent("agent_1", "Agent 1", "https://agent1.example.com")

        # Register unavailable agent
        unavailable = discovery.register_agent("agent_2", "Agent 2", "https://agent2.example.com")
        unavailable.available = False

        agents = discovery.list_agents(available_only=True)

        assert len(agents) == 1
        assert agents[0].agent_id == "agent_1"

    def test_list_agents_by_capability(self, discovery):
        """Test listing agents by capability."""
        card1 = SardisAgentCard(
            agent_id="agent_1",
            agent_name="Agent 1",
            capabilities=[AgentCapability.PAYMENT_EXECUTE],
        )
        discovery.register_agent("agent_1", "Agent 1", "https://agent1.example.com", card1)

        card2 = SardisAgentCard(
            agent_id="agent_2",
            agent_name="Agent 2",
            capabilities=[AgentCapability.CHECKOUT_CREATE],
        )
        discovery.register_agent("agent_2", "Agent 2", "https://agent2.example.com", card2)

        agents = discovery.list_agents(capability=AgentCapability.PAYMENT_EXECUTE)

        assert len(agents) == 1
        assert agents[0].agent_id == "agent_1"

    def test_list_agents_by_payment(self, discovery):
        """Test listing agents by payment support."""
        card1 = SardisAgentCard(
            agent_id="agent_1",
            agent_name="Agent 1",
            payment_capability=PaymentCapability(
                supported_tokens=["USDC"],
                supported_chains=["base"],
            ),
        )
        discovery.register_agent("agent_1", "Agent 1", "https://agent1.example.com", card1)

        card2 = SardisAgentCard(
            agent_id="agent_2",
            agent_name="Agent 2",
            payment_capability=PaymentCapability(
                supported_tokens=["USDT"],
                supported_chains=["polygon"],
            ),
        )
        discovery.register_agent("agent_2", "Agent 2", "https://agent2.example.com", card2)

        agents = discovery.list_agents(token="USDC", chain="base")

        assert len(agents) == 1
        assert agents[0].agent_id == "agent_1"

    def test_list_agents_by_token_only(self, discovery):
        """Test listing agents by token only."""
        card1 = SardisAgentCard(
            agent_id="agent_1",
            agent_name="Agent 1",
            payment_capability=PaymentCapability(
                supported_tokens=["USDC", "USDT"],
                supported_chains=["base"],
            ),
        )
        discovery.register_agent("agent_1", "Agent 1", "https://agent1.example.com", card1)

        agents = discovery.list_agents(token="USDT")

        assert len(agents) == 1

    def test_remove_agent(self, discovery):
        """Test removing an agent."""
        discovery.register_agent("agent_1", "Agent 1", "https://agent.example.com")

        result = discovery.remove_agent("https://agent.example.com")

        assert result is True
        assert discovery.get_agent("https://agent.example.com") is None

    def test_remove_agent_not_found(self, discovery):
        """Test removing non-existent agent."""
        result = discovery.remove_agent("https://nonexistent.example.com")

        assert result is False

    def test_clear_cache(self, discovery):
        """Test clearing all cached agents."""
        discovery.register_agent("agent_1", "Agent 1", "https://agent1.example.com")
        discovery.register_agent("agent_2", "Agent 2", "https://agent2.example.com")

        count = discovery.clear_cache()

        assert count == 2
        assert len(discovery.list_agents(available_only=False)) == 0



"""Tests for A2A message types."""

from datetime import datetime, timezone, timedelta

import pytest

from sardis_a2a.messages import (
    A2AMessageType,
    A2AMessageStatus,
    A2AMessage,
    A2APaymentRequest,
    A2APaymentResponse,
    A2ACredentialRequest,
    A2ACredentialResponse,
)


class TestA2AMessageType:
    """Tests for A2AMessageType enum."""

    def test_payment_types(self):
        """Test payment message types."""
        assert A2AMessageType.PAYMENT_REQUEST.value == "payment_request"
        assert A2AMessageType.PAYMENT_RESPONSE.value == "payment_response"

    def test_credential_types(self):
        """Test credential message types."""
        assert A2AMessageType.CREDENTIAL_REQUEST.value == "credential_request"
        assert A2AMessageType.CREDENTIAL_RESPONSE.value == "credential_response"

    def test_checkout_types(self):
        """Test checkout message types."""
        assert A2AMessageType.CHECKOUT_INITIATE.value == "checkout_initiate"
        assert A2AMessageType.CHECKOUT_COMPLETE.value == "checkout_complete"

    def test_status_types(self):
        """Test status message types."""
        assert A2AMessageType.ACK.value == "ack"
        assert A2AMessageType.ERROR.value == "error"


class TestA2AMessageStatus:
    """Tests for A2AMessageStatus enum."""

    def test_status_values(self):
        """Test message status values."""
        assert A2AMessageStatus.PENDING.value == "pending"
        assert A2AMessageStatus.RECEIVED.value == "received"
        assert A2AMessageStatus.PROCESSING.value == "processing"
        assert A2AMessageStatus.COMPLETED.value == "completed"
        assert A2AMessageStatus.FAILED.value == "failed"


class TestA2AMessage:
    """Tests for A2AMessage."""

    def test_create_message(self):
        """Test creating a message."""
        msg = A2AMessage(
            message_type=A2AMessageType.PAYMENT_REQUEST,
            sender_id="sender_123",
            recipient_id="recipient_456",
            payload={"amount": 1000},
        )

        assert msg.message_type == A2AMessageType.PAYMENT_REQUEST
        assert msg.sender_id == "sender_123"
        assert msg.recipient_id == "recipient_456"
        assert msg.payload == {"amount": 1000}
        assert msg.status == A2AMessageStatus.PENDING

    def test_default_values(self):
        """Test default message values."""
        msg = A2AMessage()

        assert msg.message_id is not None
        assert msg.message_type == A2AMessageType.ACK
        assert msg.status == A2AMessageStatus.PENDING
        assert msg.signature_algorithm == "Ed25519"
        assert isinstance(msg.timestamp, datetime)

    def test_correlation_fields(self):
        """Test correlation fields."""
        msg = A2AMessage(
            correlation_id="corr_123",
            in_reply_to="msg_456",
        )

        assert msg.correlation_id == "corr_123"
        assert msg.in_reply_to == "msg_456"

    def test_to_dict(self):
        """Test message serialization."""
        msg = A2AMessage(
            message_type=A2AMessageType.PAYMENT_REQUEST,
            sender_id="sender_123",
            recipient_id="recipient_456",
            payload={"amount": 1000, "token": "USDC"},
            signature="sig123",
        )

        data = msg.to_dict()

        assert data["message_type"] == "payment_request"
        assert data["sender_id"] == "sender_123"
        assert data["recipient_id"] == "recipient_456"
        assert data["payload"]["amount"] == 1000
        assert data["signature"] == "sig123"
        assert "timestamp" in data

    def test_to_dict_with_error(self):
        """Test serialization with error."""
        msg = A2AMessage(
            message_type=A2AMessageType.ERROR,
            status=A2AMessageStatus.FAILED,
            error="Payment failed",
            error_code="insufficient_funds",
        )

        data = msg.to_dict()

        assert data["message_type"] == "error"
        assert data["status"] == "failed"
        assert data["error"] == "Payment failed"
        assert data["error_code"] == "insufficient_funds"

    def test_from_dict(self):
        """Test message deserialization."""
        data = {
            "message_id": "msg_123",
            "message_type": "payment_request",
            "sender_id": "sender_123",
            "recipient_id": "recipient_456",
            "timestamp": "2024-01-15T10:30:00+00:00",
            "payload": {"amount": 1000},
            "status": "completed",
        }

        msg = A2AMessage.from_dict(data)

        assert msg.message_id == "msg_123"
        assert msg.message_type == A2AMessageType.PAYMENT_REQUEST
        assert msg.sender_id == "sender_123"
        assert msg.payload == {"amount": 1000}
        assert msg.status == A2AMessageStatus.COMPLETED

    def test_from_dict_with_expiration(self):
        """Test deserialization with expiration."""
        data = {
            "message_type": "payment_request",
            "expires_at": "2024-01-15T12:00:00+00:00",
        }

        msg = A2AMessage.from_dict(data)

        assert msg.expires_at is not None
        assert msg.expires_at.year == 2024

    def test_roundtrip_serialization(self):
        """Test serialization/deserialization roundtrip."""
        original = A2AMessage(
            message_type=A2AMessageType.CREDENTIAL_REQUEST,
            sender_id="agent_a",
            recipient_id="agent_b",
            correlation_id="corr_1",
            payload={"credential_type": "mandate"},
        )

        data = original.to_dict()
        restored = A2AMessage.from_dict(data)

        assert restored.message_type == original.message_type
        assert restored.sender_id == original.sender_id
        assert restored.recipient_id == original.recipient_id
        assert restored.correlation_id == original.correlation_id
        assert restored.payload == original.payload


class TestA2APaymentRequest:
    """Tests for A2APaymentRequest."""

    @pytest.fixture
    def payment_request(self):
        """Create a sample payment request."""
        return A2APaymentRequest(
            sender_agent_id="agent_sender",
            recipient_agent_id="agent_recipient",
            amount_minor=5000,  # $50.00
            currency="USD",
            token="USDC",
            chain="base",
            destination="0x1234567890abcdef1234567890abcdef12345678",
            purpose="Test payment",
            reference="order_123",
        )

    def test_create_payment_request(self, payment_request):
        """Test creating a payment request."""
        assert payment_request.sender_agent_id == "agent_sender"
        assert payment_request.recipient_agent_id == "agent_recipient"
        assert payment_request.amount_minor == 5000
        assert payment_request.token == "USDC"
        assert payment_request.chain == "base"
        assert payment_request.purpose == "Test payment"

    def test_default_values(self):
        """Test default payment request values."""
        req = A2APaymentRequest()

        assert req.request_id is not None
        assert req.currency == "USD"
        assert req.token == "USDC"
        assert req.chain == "base"

    def test_to_a2a_message(self, payment_request):
        """Test converting to A2A message."""
        msg = payment_request.to_a2a_message()

        assert msg.message_type == A2AMessageType.PAYMENT_REQUEST
        assert msg.sender_id == "agent_sender"
        assert msg.recipient_id == "agent_recipient"
        assert msg.correlation_id == payment_request.request_id
        assert msg.payload["amount_minor"] == 5000
        assert msg.payload["token"] == "USDC"
        assert msg.payload["chain"] == "base"
        assert msg.payload["destination"] == "0x1234567890abcdef1234567890abcdef12345678"
        assert msg.payload["purpose"] == "Test payment"
        assert msg.payload["reference"] == "order_123"

    def test_from_a2a_message(self, payment_request):
        """Test creating from A2A message."""
        msg = payment_request.to_a2a_message()
        restored = A2APaymentRequest.from_a2a_message(msg)

        assert restored.request_id == payment_request.request_id
        assert restored.sender_agent_id == payment_request.sender_agent_id
        assert restored.recipient_agent_id == payment_request.recipient_agent_id
        assert restored.amount_minor == payment_request.amount_minor
        assert restored.token == payment_request.token
        assert restored.chain == payment_request.chain
        assert restored.destination == payment_request.destination

    def test_with_callback_url(self):
        """Test payment request with callback URL."""
        req = A2APaymentRequest(
            amount_minor=1000,
            callback_url="https://callback.example.com/payments",
        )

        msg = req.to_a2a_message()

        assert msg.payload["callback_url"] == "https://callback.example.com/payments"

    def test_with_metadata(self):
        """Test payment request with metadata."""
        req = A2APaymentRequest(
            amount_minor=1000,
            metadata={"order_id": "123", "customer": "abc"},
        )

        msg = req.to_a2a_message()

        assert msg.payload["metadata"]["order_id"] == "123"
        assert msg.payload["metadata"]["customer"] == "abc"


class TestA2APaymentResponse:
    """Tests for A2APaymentResponse."""

    def test_create_success_response(self):
        """Test creating a successful payment response."""
        resp = A2APaymentResponse(
            request_id="req_123",
            sender_agent_id="agent_recipient",
            recipient_agent_id="agent_sender",
            success=True,
            status="confirmed",
            tx_hash="0xabcdef123456",
            chain="base",
            block_number=12345,
        )

        assert resp.success is True
        assert resp.status == "confirmed"
        assert resp.tx_hash == "0xabcdef123456"
        assert resp.block_number == 12345

    def test_create_failure_response(self):
        """Test creating a failed payment response."""
        resp = A2APaymentResponse(
            request_id="req_123",
            success=False,
            status="failed",
            error="Insufficient balance",
            error_code="insufficient_balance",
        )

        assert resp.success is False
        assert resp.status == "failed"
        assert resp.error == "Insufficient balance"
        assert resp.error_code == "insufficient_balance"

    def test_to_a2a_message_success(self):
        """Test converting successful response to message."""
        resp = A2APaymentResponse(
            request_id="req_123",
            success=True,
            status="confirmed",
            tx_hash="0xabcdef",
        )

        msg = resp.to_a2a_message()

        assert msg.message_type == A2AMessageType.PAYMENT_RESPONSE
        assert msg.in_reply_to == "req_123"
        assert msg.status == A2AMessageStatus.COMPLETED
        assert msg.payload["success"] is True
        assert msg.payload["tx_hash"] == "0xabcdef"

    def test_to_a2a_message_failure(self):
        """Test converting failed response to message."""
        resp = A2APaymentResponse(
            request_id="req_123",
            success=False,
            error="Payment rejected",
            error_code="rejected",
        )

        msg = resp.to_a2a_message()

        assert msg.message_type == A2AMessageType.PAYMENT_RESPONSE
        assert msg.status == A2AMessageStatus.FAILED
        assert msg.payload["success"] is False
        assert msg.error == "Payment rejected"


class TestA2ACredentialRequest:
    """Tests for A2ACredentialRequest."""

    def test_create_credential_request(self):
        """Test creating a credential request."""
        req = A2ACredentialRequest(
            sender_agent_id="agent_a",
            recipient_agent_id="agent_b",
            credential_type="mandate",
            credential_data={"mandate_id": "mand_123", "issuer": "sardis.sh"},
        )

        assert req.sender_agent_id == "agent_a"
        assert req.credential_type == "mandate"
        assert req.credential_data["mandate_id"] == "mand_123"
        assert req.verify_signature is True
        assert req.verify_expiration is True
        assert req.verify_chain is True

    def test_custom_verification_options(self):
        """Test custom verification options."""
        req = A2ACredentialRequest(
            credential_type="identity",
            verify_signature=True,
            verify_expiration=False,
            verify_chain=False,
        )

        assert req.verify_signature is True
        assert req.verify_expiration is False
        assert req.verify_chain is False

    def test_to_a2a_message(self):
        """Test converting to A2A message."""
        req = A2ACredentialRequest(
            sender_agent_id="agent_a",
            recipient_agent_id="agent_b",
            credential_type="mandate",
            credential_data={"mandate_id": "mand_123"},
        )

        msg = req.to_a2a_message()

        assert msg.message_type == A2AMessageType.CREDENTIAL_REQUEST
        assert msg.sender_id == "agent_a"
        assert msg.recipient_id == "agent_b"
        assert msg.correlation_id == req.request_id
        assert msg.payload["credential_type"] == "mandate"
        assert msg.payload["verify_signature"] is True


class TestA2ACredentialResponse:
    """Tests for A2ACredentialResponse."""

    def test_create_valid_response(self):
        """Test creating a valid credential response."""
        resp = A2ACredentialResponse(
            request_id="req_123",
            sender_agent_id="agent_b",
            recipient_agent_id="agent_a",
            valid=True,
            signature_valid=True,
            not_expired=True,
            chain_valid=True,
            verification_details={"verified_fields": ["issuer", "subject"]},
        )

        assert resp.valid is True
        assert resp.signature_valid is True
        assert resp.not_expired is True
        assert resp.chain_valid is True

    def test_create_invalid_response(self):
        """Test creating an invalid credential response."""
        resp = A2ACredentialResponse(
            request_id="req_123",
            valid=False,
            signature_valid=False,
            error="Invalid signature",
            error_code="invalid_signature",
        )

        assert resp.valid is False
        assert resp.signature_valid is False
        assert resp.error == "Invalid signature"

    def test_to_a2a_message_valid(self):
        """Test converting valid response to message."""
        resp = A2ACredentialResponse(
            request_id="req_123",
            valid=True,
            signature_valid=True,
            not_expired=True,
        )

        msg = resp.to_a2a_message()

        assert msg.message_type == A2AMessageType.CREDENTIAL_RESPONSE
        assert msg.in_reply_to == "req_123"
        assert msg.status == A2AMessageStatus.COMPLETED
        assert msg.payload["valid"] is True

    def test_to_a2a_message_invalid(self):
        """Test converting invalid response to message."""
        resp = A2ACredentialResponse(
            request_id="req_123",
            valid=False,
            error="Mandate expired",
        )

        msg = resp.to_a2a_message()

        assert msg.message_type == A2AMessageType.CREDENTIAL_RESPONSE
        assert msg.status == A2AMessageStatus.FAILED
        assert msg.payload["valid"] is False



"""Agent discovery service for A2A protocol.

Discovers and caches agent cards from:
- /.well-known/agent-card.json endpoints
- Manual registrations
"""

from __future__ import annotations

import logging
from dataclasses import dataclass, field
from datetime import datetime, timezone, timedelta
from typing import Any, Dict, List, Optional, Protocol

from .agent_card import SardisAgentCard, AgentCapability, PaymentCapability

logger = logging.getLogger(__name__)


@dataclass(slots=True)
class DiscoveredAgent:
    """A discovered agent with cached card data."""

    agent_id: str
    agent_name: str
    agent_url: str  # Base URL for the agent

    # Cached agent card
    card: Optional[SardisAgentCard] = None
    card_raw: Optional[Dict[str, Any]] = None

    # Cache metadata
    discovered_at: datetime = field(default_factory=lambda: datetime.now(timezone.utc))
    last_verified_at: datetime = field(default_factory=lambda: datetime.now(timezone.utc))
    cache_ttl_seconds: int = 3600  # 1 hour default

    # Status
    available: bool = True
    last_error: Optional[str] = None

    def is_cache_valid(self) -> bool:
        """Check if the cached data is still valid."""
        expiry = self.last_verified_at + timedelta(seconds=self.cache_ttl_seconds)
        return datetime.now(timezone.utc) < expiry

    def supports_capability(self, capability: AgentCapability) -> bool:
        """Check if this agent supports a specific capability."""
        if self.card:
            return self.card.supports_capability(capability)
        return False

    def supports_payment(self, token: str, chain: str) -> bool:
        """Check if this agent supports a specific payment type."""
        if self.card:
            return self.card.supports_token(token) and self.card.supports_chain(chain)
        return False


class HttpClient(Protocol):
    """Protocol for HTTP client operations."""

    async def get(self, url: str, headers: Dict[str, str] | None = None) -> tuple[int, Dict[str, Any]]:
        """Make an HTTP GET request.

        Returns:
            Tuple of (status_code, json_response)
        """
        ...


class AgentDiscoveryService:
    """
    Service for discovering and caching A2A agent cards.

    Discovery methods:
    1. Fetch /.well-known/agent-card.json from agent URLs
    2. Manual registration of known agents
    3. UCP profiles at /.well-known/ucp.json (fallback)

    Features:
    - TTL-based caching (1 hour default)
    - Capability filtering
    - Payment capability validation
    """

    def __init__(
        self,
        http_client: HttpClient | None = None,
        cache_ttl_seconds: int = 3600,
    ) -> None:
        """
        Initialize the discovery service.

        Args:
            http_client: HTTP client for fetching agent cards
            cache_ttl_seconds: Cache TTL in seconds (default: 1 hour)
        """
        self._http_client = http_client
        self._cache_ttl = cache_ttl_seconds
        self._agents: Dict[str, DiscoveredAgent] = {}

    async def discover_agent(
        self,
        agent_url: str,
        force_refresh: bool = False,
    ) -> DiscoveredAgent:
        """
        Discover an agent from its base URL.

        Fetches the agent card from /.well-known/agent-card.json

        Args:
            agent_url: Base URL of the agent (e.g., https://agent.example.com)
            force_refresh: Force refresh even if cache is valid

        Returns:
            DiscoveredAgent with cached card data
        """
        # Normalize URL
        agent_url = agent_url.rstrip("/")

        # Check cache
        if agent_url in self._agents and not force_refresh:
            cached = self._agents[agent_url]
            if cached.is_cache_valid():
                logger.debug(f"Using cached agent card for {agent_url}")
                return cached

        # Fetch agent card
        agent_card_url = f"{agent_url}/.well-known/agent-card.json"

        logger.info(f"Discovering agent at {agent_card_url}")

        if self._http_client is None:
            # Create a mock agent for testing without HTTP client
            agent = DiscoveredAgent(
                agent_id=f"agent_{hash(agent_url) % 10000}",
                agent_name=f"Agent at {agent_url}",
                agent_url=agent_url,
                cache_ttl_seconds=self._cache_ttl,
                available=False,
                last_error="HTTP client not configured",
            )
            self._agents[agent_url] = agent
            return agent

        try:
            status, data = await self._http_client.get(agent_card_url)

            if status != 200:
                raise Exception(f"HTTP {status} from {agent_card_url}")

            # Parse agent card
            card = self._parse_agent_card(data)

            agent = DiscoveredAgent(
                agent_id=card.agent_id,
                agent_name=card.agent_name,
                agent_url=agent_url,
                card=card,
                card_raw=data,
                cache_ttl_seconds=self._cache_ttl,
                available=True,
            )

            self._agents[agent_url] = agent

            logger.info(
                f"Discovered agent: id={card.agent_id}, name={card.agent_name}, "
                f"capabilities={len(card.capabilities)}"
            )

            return agent

        except Exception as e:
            logger.warning(f"Failed to discover agent at {agent_url}: {e}")

            # Return or update with error status
            if agent_url in self._agents:
                self._agents[agent_url].available = False
                self._agents[agent_url].last_error = str(e)
                return self._agents[agent_url]

            agent = DiscoveredAgent(
                agent_id=f"unknown_{hash(agent_url) % 10000}",
                agent_name=f"Unknown Agent",
                agent_url=agent_url,
                cache_ttl_seconds=self._cache_ttl,
                available=False,
                last_error=str(e),
            )
            self._agents[agent_url] = agent
            return agent

    def register_agent(
        self,
        agent_id: str,
        agent_name: str,
        agent_url: str,
        card: SardisAgentCard | None = None,
    ) -> DiscoveredAgent:
        """
        Manually register an agent.

        Args:
            agent_id: Unique agent identifier
            agent_name: Display name
            agent_url: Base URL
            card: Optional pre-populated agent card

        Returns:
            Registered DiscoveredAgent
        """
        agent_url = agent_url.rstrip("/")

        agent = DiscoveredAgent(
            agent_id=agent_id,
            agent_name=agent_name,
            agent_url=agent_url,
            card=card,
            cache_ttl_seconds=self._cache_ttl,
            available=True,
        )

        self._agents[agent_url] = agent

        logger.info(f"Registered agent: id={agent_id}, name={agent_name}, url={agent_url}")

        return agent

    def get_agent(self, agent_url: str) -> Optional[DiscoveredAgent]:
        """Get a cached agent by URL."""
        return self._agents.get(agent_url.rstrip("/"))

    def get_agent_by_id(self, agent_id: str) -> Optional[DiscoveredAgent]:
        """Get a cached agent by ID."""
        for agent in self._agents.values():
            if agent.agent_id == agent_id:
                return agent
        return None

    def list_agents(
        self,
        capability: AgentCapability | None = None,
        token: str | None = None,
        chain: str | None = None,
        available_only: bool = True,
    ) -> List[DiscoveredAgent]:
        """
        List discovered agents with optional filtering.

        Args:
            capability: Filter by required capability
            token: Filter by supported token
            chain: Filter by supported chain
            available_only: Only return available agents

        Returns:
            List of matching agents
        """
        agents = list(self._agents.values())

        if available_only:
            agents = [a for a in agents if a.available]

        if capability:
            agents = [a for a in agents if a.supports_capability(capability)]

        if token and chain:
            agents = [a for a in agents if a.supports_payment(token, chain)]
        elif token:
            agents = [a for a in agents if a.card and a.card.supports_token(token)]
        elif chain:
            agents = [a for a in agents if a.card and a.card.supports_chain(chain)]

        return agents

    def remove_agent(self, agent_url: str) -> bool:
        """Remove an agent from the cache."""
        agent_url = agent_url.rstrip("/")
        if agent_url in self._agents:
            del self._agents[agent_url]
            return True
        return False

    def clear_cache(self) -> int:
        """Clear all cached agents. Returns count of removed agents."""
        count = len(self._agents)
        self._agents.clear()
        return count

    def _parse_agent_card(self, data: Dict[str, Any]) -> SardisAgentCard:
        """Parse an agent card from JSON data."""
        # Parse capabilities
        capabilities = []
        for cap_str in data.get("capabilities", []):
            try:
                capabilities.append(AgentCapability(cap_str))
            except ValueError:
                logger.debug(f"Unknown capability: {cap_str}")

        # Parse payment capability
        payment_data = data.get("payment", {})
        payment_cap = PaymentCapability(
            supported_tokens=payment_data.get("supported_tokens", ["USDC"]),
            supported_chains=payment_data.get("supported_chains", ["base"]),
            min_amount_minor=payment_data.get("min_amount_minor", 100),
            max_amount_minor=payment_data.get("max_amount_minor", 100_000_00),
            ap2_compliant=payment_data.get("ap2_compliant", True),
            x402_compliant=payment_data.get("x402_compliant", True),
            ucp_compliant=payment_data.get("ucp_compliant", True),
        )

        return SardisAgentCard(
            agent_id=data.get("agent_id", ""),
            agent_name=data.get("name", ""),
            agent_version=data.get("version", "1.0.0"),
            agent_description=data.get("description", ""),
            operator_name=data.get("operator", {}).get("name", ""),
            operator_url=data.get("operator", {}).get("url", ""),
            operator_contact=data.get("operator", {}).get("contact"),
            capabilities=capabilities,
            payment_capability=payment_cap,
        )


__all__ = [
    "DiscoveredAgent",
    "HttpClient",
    "AgentDiscoveryService",
]



"""Sardis Agent Card for A2A protocol.

The agent card is published at /.well-known/agent-card.json and declares
the agent's capabilities for other agents to discover.

Sardis agent cards declare:
- Payment capabilities (tokens, chains, protocols)
- Service endpoints (REST, MCP, webhooks)
- Supported message types
"""

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime, timezone
from enum import Enum
from typing import Any, Dict, List, Optional


class AgentCapability(str, Enum):
    """Capabilities that a Sardis agent supports."""

    # Payment capabilities
    PAYMENT_EXECUTE = "payment.execute"  # Execute AP2 payment mandates
    PAYMENT_VERIFY = "payment.verify"  # Verify mandate chains
    PAYMENT_REFUND = "payment.refund"  # Process refunds

    # Mandate capabilities
    MANDATE_INGEST = "mandate.ingest"  # Receive mandates
    MANDATE_SIGN = "mandate.sign"  # Sign mandates

    # Wallet capabilities
    WALLET_BALANCE = "wallet.balance"  # Check wallet balances
    WALLET_HOLD = "wallet.hold"  # Create/manage holds

    # Checkout capabilities (UCP)
    CHECKOUT_CREATE = "checkout.create"  # Create UCP checkouts
    CHECKOUT_COMPLETE = "checkout.complete"  # Complete checkouts

    # Micropayment capabilities
    X402_MICROPAY = "x402.micropay"  # x402 micropayments


@dataclass(slots=True)
class PaymentCapability:
    """Payment-specific capability declaration."""

    # Supported tokens
    supported_tokens: List[str] = field(
        default_factory=lambda: ["USDC", "USDT", "PYUSD", "EURC"]
    )

    # Supported chains
    supported_chains: List[str] = field(
        default_factory=lambda: ["base", "polygon", "ethereum", "arbitrum", "optimism"]
    )

    # Limits (in minor units)
    min_amount_minor: int = 100  # $1.00
    max_amount_minor: int = 100_000_00  # $100,000.00

    # Protocol compliance
    ap2_compliant: bool = True
    x402_compliant: bool = True
    ucp_compliant: bool = True

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for serialization."""
        return {
            "supported_tokens": self.supported_tokens,
            "supported_chains": self.supported_chains,
            "min_amount_minor": self.min_amount_minor,
            "max_amount_minor": self.max_amount_minor,
            "ap2_compliant": self.ap2_compliant,
            "x402_compliant": self.x402_compliant,
            "ucp_compliant": self.ucp_compliant,
        }


@dataclass(slots=True)
class ServiceEndpoint:
    """Service endpoint for agent communication."""

    url: str
    protocol: str = "https"  # https, wss, mcp
    auth_required: bool = False
    auth_type: Optional[str] = None  # bearer, api_key, signature

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for serialization."""
        return {
            "url": self.url,
            "protocol": self.protocol,
            "auth_required": self.auth_required,
            "auth_type": self.auth_type,
        }


@dataclass(slots=True)
class SardisAgentCard:
    """
    Sardis Agent Card for A2A protocol.

    Published at /.well-known/agent-card.json to allow other agents
    to discover and interact with this Sardis instance.

    Conforms to the A2A agent card specification.
    """

    # Agent identity
    agent_id: str
    agent_name: str
    agent_version: str = "1.0.0"
    agent_description: str = ""

    # Operator information
    operator_name: str = "Sardis"
    operator_url: str = "https://sardis.sh"
    operator_contact: Optional[str] = None

    # Capabilities
    capabilities: List[AgentCapability] = field(
        default_factory=lambda: [
            AgentCapability.PAYMENT_EXECUTE,
            AgentCapability.PAYMENT_VERIFY,
            AgentCapability.MANDATE_INGEST,
            AgentCapability.WALLET_BALANCE,
            AgentCapability.CHECKOUT_CREATE,
            AgentCapability.CHECKOUT_COMPLETE,
            AgentCapability.X402_MICROPAY,
        ]
    )
    payment_capability: PaymentCapability = field(default_factory=PaymentCapability)

    # Service endpoints
    api_endpoint: Optional[ServiceEndpoint] = None
    mcp_endpoint: Optional[str] = None  # MCP server command/args
    webhook_endpoint: Optional[ServiceEndpoint] = None
    a2a_endpoint: Optional[ServiceEndpoint] = None

    # Verification
    signing_key_id: Optional[str] = None
    public_key: Optional[str] = None
    key_algorithm: str = "Ed25519"

    # Metadata
    created_at: datetime = field(default_factory=lambda: datetime.now(timezone.utc))
    updated_at: datetime = field(default_factory=lambda: datetime.now(timezone.utc))
    metadata: Dict[str, Any] = field(default_factory=dict)

    def supports_capability(self, capability: AgentCapability) -> bool:
        """Check if this agent supports a specific capability."""
        return capability in self.capabilities

    def supports_token(self, token: str) -> bool:
        """Check if this agent supports a specific token."""
        return token.upper() in self.payment_capability.supported_tokens

    def supports_chain(self, chain: str) -> bool:
        """Check if this agent supports a specific chain."""
        return chain.lower() in self.payment_capability.supported_chains

    def to_dict(self) -> Dict[str, Any]:
        """
        Convert to dictionary for JSON serialization.

        This is the format served at /.well-known/agent-card.json
        """
        card = {
            # A2A spec fields
            "agent_id": self.agent_id,
            "name": self.agent_name,
            "version": self.agent_version,
            "description": self.agent_description,
            # Operator
            "operator": {
                "name": self.operator_name,
                "url": self.operator_url,
                "contact": self.operator_contact,
            },
            # Capabilities
            "capabilities": [cap.value for cap in self.capabilities],
            "payment": self.payment_capability.to_dict(),
            # Endpoints
            "endpoints": {},
            # Verification
            "signing": {
                "key_id": self.signing_key_id,
                "public_key": self.public_key,
                "algorithm": self.key_algorithm,
            } if self.signing_key_id else None,
            # Metadata
            "created_at": self.created_at.isoformat(),
            "updated_at": self.updated_at.isoformat(),
            "metadata": self.metadata,
        }

        # Add endpoints
        if self.api_endpoint:
            card["endpoints"]["api"] = self.api_endpoint.to_dict()
        if self.mcp_endpoint:
            card["endpoints"]["mcp"] = self.mcp_endpoint
        if self.webhook_endpoint:
            card["endpoints"]["webhook"] = self.webhook_endpoint.to_dict()
        if self.a2a_endpoint:
            card["endpoints"]["a2a"] = self.a2a_endpoint.to_dict()

        return card


def create_sardis_agent_card(
    agent_id: str,
    agent_name: str,
    api_base_url: str,
    mcp_command: Optional[str] = None,
    signing_key_id: Optional[str] = None,
    public_key: Optional[str] = None,
) -> SardisAgentCard:
    """
    Create a standard Sardis agent card with default capabilities.

    Args:
        agent_id: Unique agent identifier
        agent_name: Display name for the agent
        api_base_url: Base URL for the Sardis API
        mcp_command: Optional MCP server command
        signing_key_id: Optional signing key ID
        public_key: Optional public key for verification

    Returns:
        Configured SardisAgentCard
    """
    return SardisAgentCard(
        agent_id=agent_id,
        agent_name=agent_name,
        agent_description="Sardis Payment Agent - secure AI payment infrastructure",
        api_endpoint=ServiceEndpoint(
            url=f"{api_base_url}/api/v2",
            protocol="https",
            auth_required=True,
            auth_type="bearer",
        ),
        mcp_endpoint=mcp_command,
        a2a_endpoint=ServiceEndpoint(
            url=f"{api_base_url}/api/v2/a2a",
            protocol="https",
            auth_required=True,
            auth_type="signature",
        ),
        signing_key_id=signing_key_id,
        public_key=public_key,
    )


__all__ = [
    "AgentCapability",
    "PaymentCapability",
    "ServiceEndpoint",
    "SardisAgentCard",
    "create_sardis_agent_card",
]
